# Copyright (c) 2023 Philip May
# This software is distributed under the terms of the MIT license
# which is available at https://opensource.org/licenses/MIT

"""Scraper package."""

from abc import ABC, abstractmethod
from typing import Sequence

from attr import define

from spider_scrape.db import DataManager


@define
class SpiderScraper(ABC):
    data_manager: DataManager

    @abstractmethod
    def process_batch(self, batch: Sequence):
        pass

    def run(self):
        while True:
            batch = self.data_manager.load_batch()
            if len(batch) == 0:
                break
            new_batch = self.process_batch(batch)
            if len(new_batch) > 0:
                self.data_manager.save_batch(new_batch)



# Copyright (c) 2023 Philip May
# This software is distributed under the terms of the MIT license
# which is available at https://opensource.org/licenses/MIT

"""DB tools package."""

from abc import ABC, abstractmethod
from typing import Sequence

from attr import define


@define
class DataManager(ABC):
    @abstractmethod
    def load_batch(self) -> Sequence:
        pass

    @abstractmethod
    def save_batch(self, batch: Sequence):
        pass



# This software is distributed under the terms of the MIT license
# which is available at https://opensource.org/licenses/MIT

"""ArangoDB tools package."""

from contextlib import closing
from typing import Optional, Sequence, Union

from arango import ArangoClient  # type: ignore
from attr import define, field

from spider_scrape.db import DataManager


@define
class ArangoDataManager(DataManager):
    hosts: Union[str, Sequence[str]]
    db_name: str
    username: str
    password: str
    collection_name: str
    attribute_name: str
    batch_size: int = field(default=20)
    aql_overwrite: Optional[str] = field(default=None)

    def get_arango_client(self):
        arango_client = ArangoClient(hosts=self.hosts)
        return arango_client

    def get_connection(self, arango_client):
        connection = arango_client.db(self.db_name, username=self.username, password=self.password)
        return connection

    def load_batch(self) -> Sequence:
        with closing(self.get_arango_client()) as arango_client:
            connection = self.get_connection(arango_client)
            bind_vars = {
                "@coll": self.collection_name,
                "attribute": self.attribute_name,
                "batch_size": self.batch_size,
            }
            if self.aql_overwrite is None:
                aql = "FOR doc IN @@coll FILTER !HAS(doc, @attribute) LIMIT @batch_size RETURN doc"
            else:
                aql = self.aql_overwrite
            cursor = connection.aql.execute(
                aql,
                bind_vars=bind_vars,
                batch_size=self.batch_size,
            )
            with closing(cursor) as closing_cursor:
                batch = closing_cursor.batch()
        return batch

    def save_batch(self, batch: Sequence):
        with closing(self.get_arango_client()) as arango_client:
            connection = self.get_connection(arango_client)
            collection = connection.collection(self.collection_name)
            collection.import_bulk(batch, on_duplicate="update")



# Copyright (c) 2023 Philip May
# This software is distributed under the terms of the MIT license
# which is available at https://opensource.org/licenses/MIT

"""Beautiful Soup specific tools."""

import re
from typing import Optional

import mdformat
from markdownify import MarkdownConverter, markdownify  # type: ignore


def extract_text(soup, join_str=None) -> str:
    if join_str is None:
        join_str = " "
    texts = [text for text in soup.stripped_strings]
    result: str = join_str.join(texts)
    return result


def normalize_text(text) -> str:
    text = text.replace("\xa0", " ")
    text = text.replace("\u200b", " ")
    text = re.sub("  +", " ", text)
    text = text.strip()
    return text


def extract_one(soup, name=None, attrs={}, **kwargs):
    result = soup.find_all(name, attrs, **kwargs)
    assert len(result) == 1, len(result)
    result = result[0]
    return result


def extract_all(soup, name=None, attrs={}, **kwargs):
    result = soup.find_all(name, attrs, **kwargs)
    return result


def remove_all(soup, name=None, attrs={}, **kwargs) -> None:
    result = soup.find_all(name, attrs, **kwargs)
    for r in result:
        r.decompose()


def soup_to_md(soup, mdformat_options: Optional[dict] = None) -> str:
    if mdformat_options is None:
        mdformat_options = {"number": True, "wrap": "no"}
    text = MarkdownConverter().convert_soup(soup)
    text = mdformat.text(text, options=mdformat_options)
    return text


def html_to_md(html, mdformat_options: Optional[dict] = None) -> str:
    if mdformat_options is None:
        mdformat_options = {"number": True, "wrap": "no"}
    text = markdownify(html)
    text = mdformat.text(text, options=mdformat_options)
    return text
